/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/

package org.esupportail.portal.channels.CWebdav;


/**
 * Id: Config.java,v 1.0
 * Copyright (c) 2004 Esup Portail (www.esup-portail.org)
 * Classes: Config
 * Original Author: Sbastien GAUDIN
 * This class parse the CWebdav.xml config file to return all parameters
 */



import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import org.jasig.portal.services.LogService;
import org.apache.commons.digester.Digester;
import org.esupportail.portal.channels.CWebdav.exception.PropertiesException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class Config { 
	
	/*Server*/
	private String url;
	//private String externUrl;
	//private String externText;
	
	/*Authentication*/
	private String usernameAttribute;
	private String trustedLogin;
	private String trustedPassword;
	//private String classAuth;
	
	/*classAccess*/
	private String classAccess;
	
	/*Interface options*/
	private boolean confirmDel = true;
	private boolean allowDelNonEmptyFolder = true;
	
	/*Data base*/
	//private String prefStoreTable;
	//private String pool;
	
	/*Regular expression for "invisibles" files*/
	private Vector invisibleRegexp = new Vector();
	
	private String delete = null;
	private String rename = null;
	private String newDir = null;
	private String copy = null;
	private String move = null;

	/*path of the config file*/
	private String configFilePath ="/properties/channels/org_esup/CWebdav/CWebDav.xml";
	
	private static Config singleton = null;

	public static Config getInstance() throws PropertiesException {
		if (singleton == null) {
			singleton = new Config();
		}
		return singleton;

	}
	
	/**
	 * @param configFilePath
	 */
	private Config() throws PropertiesException {

		Digester digester = new Digester();
		digester.setValidating(false);
		digester.push(this);

		/*The server
		 * 
		 * Add url, externUrl and externText
		 * */		
		digester.addSetProperties("WEBDAV/SERVER");
		
		/*The authentication
		 * 
		 * Add username_attribute,trusted_password and classAuth
		 * */		
		/**/digester.addSetProperties("WEBDAV/AUTH");
		digester.addCallMethod("WEBDAV/AUTH", "setClassAuth", 1);
		digester.addCallParam("WEBDAV/AUTH", 0, "class");
		
		/*
		 * 
		 * Add classAccess
		 * */		
		digester.addCallMethod("WEBDAV/DAV_ACCESS", "setClassAccess", 1);
		digester.addCallParam("WEBDAV/DAV_ACCESS", 0, "class");
		
		/*Interface options
		 * 
		 * Add confirmDel and allowDelNonEmptyFolder
		 * */		
		digester.addCallMethod("WEBDAV/DEFAULT_OPTIONS_INTERFACE", "setConfirmDel", 1);
		digester.addCallParam("WEBDAV/DEFAULT_OPTIONS_INTERFACE", 0, "confirmDel");
		digester.addCallMethod("WEBDAV/DEFAULT_OPTIONS_INTERFACE", "setAllowDelNonEmptyFolder", 1);
		digester.addCallParam("WEBDAV/DEFAULT_OPTIONS_INTERFACE", 0, "allowDelNonEmptyFolder");

		  
		/*Invisibles files
		 * 
		 * Add every  regular Expression
		 * */		
		digester.addCallMethod("WEBDAV/INVISIBLE_FILES/REG_EXP", "addInvisibleRegexp", 1);
		digester.addCallParam("WEBDAV/INVISIBLE_FILES/REG_EXP", 0, "pattern");

		digester.addSetProperties( "WEBDAV/TEXT_ACTION");
		
		/*Data base
		 * 
		 * Add prefStoreTable and pool
		 * */		
		digester.addSetProperties("WEBDAV/DB");
		
		URL url = Config.class.getResource(configFilePath);		
		
		if (url != null) {

			InputSource inSource = new InputSource(url.toExternalForm());

			try {
				//parse config file
				digester.parse(inSource);
			} catch (IOException e) {
				LogService.log(
					LogService.ERROR,
					"CWebdav::config::config() : Unable to open config file"
						+ configFilePath
						+ "\n"
						+ e.toString());
				throw new PropertiesException();
			} catch (SAXException e) {
				LogService.log(
					LogService.ERROR,
					"CWebdav::config::config() : SAXException"
						+ e.toString());
				throw new PropertiesException();
			}
		} else { 
			LogService.log(
				LogService.ERROR,
				"CWebdav::config::config() : "
					+ configFilePath
					+ " does not exist.\n");
			throw new PropertiesException();
		}
	}
	
	/**
	 * @return Return string that define the external server
	 */
	/*public String getExternText() throws PropertiesException {
		if (externText==null)
			throw new PropertiesException();
		return externText;
	}*/
	
	
	/**
	 * @param string that define the external server
	 */
	/*public void setExternText(String externText) {
		this.externText = externText;
	}*/
	/**
	 * @return Returns the externUrl.
	 */
	/*public String getExternUrl() throws PropertiesException {
		if (externUrl==null)
			throw new PropertiesException();
		return externUrl;
	}*/
	/**
	 * @param externUrl The externUrl to set.
	 */
	/*public void setExternUrl(String externUrl) {
		this.externUrl = externUrl;
	}*/
	/**
	 * @return Returns the url.
	 */
	public String getUrl() throws PropertiesException {
		if (url==null)
			throw new PropertiesException();
		return url;
	}
	/**
	 * @param url The url to set.
	 */
	public void setUrl(String url) {
		this.url = url; 
	}
	/**
	 * @return Returns the allowDelNonEmptyFolder.
	 */
	public boolean getAllowDelNonEmptyFolder() {
		return allowDelNonEmptyFolder;
	}
	/**
	 * @param allowDelNonEmptyFolder The allowDelNonEmptyFolder to set.
	 */
	public void setAllowDelNonEmptyFolder(String allowDelNonEmptyFolder) {
		if (allowDelNonEmptyFolder.equals("true"))
			this.allowDelNonEmptyFolder = true;
		else 
			this.allowDelNonEmptyFolder = false;
	}
	/**
	 * @return Returns the classAccess.
	 */
	public String getClassAccess() throws PropertiesException {
		if (classAccess==null)
			throw new PropertiesException();
		return classAccess;
	}
	/**
	 * @param classAccess The classAccess to set.
	 */
	public void setClassAccess(String classAccess) {
		this.classAccess = classAccess;
	}
	/**
	 * @return Returns the classAuth.
	 */
	/*public String getClassAuth() throws PropertiesException {
		if (classAuth==null)
			throw new PropertiesException();
		return classAuth;
	}*/
	/**
	 * @param classAuth The classAuth to set.
	 */
	/*public void setClassAuth(String classAuth) {
		this.classAuth = classAuth;
	}*/
	/**
	 * @return Returns the confirmDel.
	 */
	public boolean getConfirmDel() {
		return confirmDel;
	}
	/**
	 * @param confirmDel The confirmDel to set.
	 */
	public void setConfirmDel(String confirmDel) {
		if (confirmDel.equals("true"))
			this.confirmDel = true;
		else 
			this.confirmDel = false;
	}
	/**
	 * @return Returns the pool.
	 */
	/*public String getPool() throws PropertiesException {
		if (pool==null)
			throw new PropertiesException();
		return pool;
	}*/
	/**
	 * @param pool The pool to set.
	 */
	/*public void setPool(String pool) {
		this.pool = pool;
	}*/
	/**
	 * @return Returns the prefStoreTable.
	 */
	/*public String getPrefStoreTable() throws PropertiesException {
		if (prefStoreTable==null)
			throw new PropertiesException();
		return prefStoreTable;
	}*/
	/**
	 * @param prefStoreTable The prefStoreTable to set.
	 */
	/*public void setPrefStoreTable(String prefStoreTable) {
		this.prefStoreTable = prefStoreTable;
	} */  
	
	/**
	 * @return Returns the trustedLogin.
	 */
	public String getTrustedLogin() {
		return trustedLogin;
	}
	/**
	 * @param trustedLogin The trustedLogin to set.
	 */
	public void setTrustedLogin(String trustedLogin) {
		this.trustedLogin = trustedLogin;
	}
	
	/**
	 * @return Returns the trustedPassword.
	 */
	public String getTrustedPassword() throws PropertiesException {
		if (trustedPassword==null)
			throw new PropertiesException();
		return trustedPassword;
	}
	/**
	 * @param trustedPassword The trustedPassword to set.
	 */
	public void setTrustedPassword(String trustedPassword) {
		this.trustedPassword = trustedPassword;
	}
	/**
	 * @return Returns the usernameAttribute.
	 */
	public String getUsernameAttribute() throws PropertiesException {
		if (usernameAttribute==null)
			throw new PropertiesException();
		return usernameAttribute;
	}
	/**
	 * @param usernameAttribute The usernameAttribute to set.
	 */
	public void setUsernameAttribute(String usernameAttribute) {
		this.usernameAttribute = usernameAttribute;
	}
	/**
	 * @return Returns the invisibleRexexp.
	 */
	public Vector getInvisibleRegexp() throws PropertiesException {
		if (invisibleRegexp==null)
			throw new PropertiesException();
		return this.invisibleRegexp;
	}
	/**
	 * @param invisibleRexexp The invisibleRexexp to set.
	 */
	public void addInvisibleRegexp(String regexp) {
		this.invisibleRegexp.add(regexp);
		
	}
	/**
	 * @return Returns the copy.
	 */
	public String getCopy() throws PropertiesException {
		if (copy==null)
			throw new PropertiesException();
		return copy;
	}
	/**
	 * @param copy The copy to set.
	 */
	public void setCopy(String copy) {
		this.copy = copy;
	}
	/**
	 * @return Returns the delete.
	 */
	public String getDelete() throws PropertiesException {
		if (delete==null)
			throw new PropertiesException();
		return delete;
	}
	/**
	 * @param delete The delete to set.
	 */
	public void setDelete(String delete) {
		this.delete = delete;
	}
	/**
	 * @return Returns the move.
	 */
	public String getMove() throws PropertiesException {
		if (move==null)
			throw new PropertiesException();
		return move;
	}
	/**
	 * @param move The move to set.
	 */
	public void setMove(String move) {
		this.move = move;
	}
	/**
	 * @return Returns the newDir.
	 */
	public String getNewDir() throws PropertiesException {
		if (newDir==null)
			throw new PropertiesException();
		return newDir;
	}
	/**
	 * @param newDir The newDir to set.
	 */
	public void setNewDir(String newDir) {
		this.newDir = newDir;
	}
	/**
	 * @return Returns the rename.
	 */
	public String getRename() throws PropertiesException {
		if (rename==null)
			throw new PropertiesException();
		return rename;
	}
	/**
	 * @param rename The rename to set.
	 */
	public void setRename(String rename) {
		this.rename = rename;
	}
}
