/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/

package org.esupportail.portal.channels.CWebdav.provider;
 
import java.io.IOException; 
import java.io.InputStream; 
import java.net.MalformedURLException;
import java.util.Vector; 
import org.apache.commons.httpclient.HttpException;
import org.apache.util.HttpURL;
import org.apache.webdav.lib.WebdavResource;
import org.esupportail.portal.channels.CWebdav.exception.ApplicationException;
import org.esupportail.portal.channels.CWebdav.exception.BadFormatException;
import org.esupportail.portal.channels.CWebdav.exception.CopyException;
import org.esupportail.portal.channels.CWebdav.exception.CreateDirectoryException;
import org.esupportail.portal.channels.CWebdav.exception.DeleteException;
import org.esupportail.portal.channels.CWebdav.exception.EmptyFileOnUploadException;
import org.esupportail.portal.channels.CWebdav.exception.MoveException;
import org.esupportail.portal.channels.CWebdav.exception.NotExistsResourceException;
import org.esupportail.portal.channels.CWebdav.exception.OverQuotaException;
import org.esupportail.portal.channels.CWebdav.exception.PasteDeletedResourceException;
import org.esupportail.portal.channels.CWebdav.exception.PasteInChildDirectoryException;
import org.esupportail.portal.channels.CWebdav.exception.PasteWithSameNameException;
import org.esupportail.portal.channels.CWebdav.exception.RenameException;
import org.esupportail.portal.channels.CWebdav.exception.ServerException;
import org.esupportail.portal.channels.CWebdav.exception.StillExistsException;
import org.esupportail.portal.channels.CWebdav.exception.UploadException;
import org.jasig.portal.MultipartDataSource;
import org.jasig.portal.services.LogService;

/**
 * Id: DavAccessImpl.java,v 1.0 14 avr. 2004
 * Copyright (c) 2004 Esup Portail (www.esup-portail.org)
 * Classes: DavAccessImpl
 * Original Author: Yohan Colmant
 * 
 */
public class DavAccessImpl extends DavAccess{
	
	/**
	 * The dav access uri
	 */
	private String uri;
	
	/**
	 * The user login
	 */
	private String login;
	
	/**
	 * The password for the dav acccess
	 */
	private String password;
	
	/**
	 * The Http url used to access the webdav
	 */
	private HttpURL httpURL;
	
	
	/** 
	 * The WebDAV ressource 
	 */
	private WebdavResource resource;
	
	/**
	 * The resource used for the file to download
	 */
	private WebdavResource fileToDownload = null;
	
    
    
    
    
	
	/**
     * Default constructor
     */
	public DavAccessImpl() {			
	}
	
	
	/**
	 * Give the user login
	 * @return the user login
	 */
	public String getLogin() {
		return this.login;
	}
	
	/**
	 * Give the user password
	 * @return the user password
	 */
	public String getPassword() {
		return this.password;
	}
	
	/**
	 * Give the uri webdav access
	 * @return the uri webdav access
	 */
	public String getUri() {
		return this.uri;
	}
	
	/**
     * This method initialize the parameters. You have to call this method after calling connect method.
     * @param uri webdav uri
     * @param login connexion login
     * @param password connexion password
     * @throws MalformedURLException
     */
	public void init(String uri, String login, String password) throws MalformedURLException {	
		this.uri = uri;	
		if (this.uri.charAt(this.uri.length()-1)!='/')
			this.uri = this.uri+"/";
		this.login = login;
		this.password = password;
		httpURL = new HttpURL(this.uri);
        httpURL.setUserInfo(this.login, this.password);
	}
	
	/**
	 * Connect this object to the server	 
     * @throws ServerException
	 */
	public void connect() throws ServerException {		
		try {
			// connection
			if (this.resource == null) {
				this.resource = new WebdavResource(httpURL);
				this.resource.setDebug(0);
			}
			else this.resource.setHttpURL(httpURL);
		}
		catch (IOException ex) {  
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::connect "+ex);
            throw new ServerException();
        }
		catch (HttpException ex) {  
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::connect "+ex);
            throw new ServerException();
        }
    }
	
	
	/**
	 * Disconnection of the server
	 * @throws ServerException
	 */
	public void disconnect() throws ServerException {
		try {
			this.resource.close();			
		}
		catch (IOException ex) {  
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::disconnect "+ex);
            throw new ServerException();
        }
    }
	
	
	/**
	 * All ressources for a given path
	 * @param path directory path on the dav server
	 * @return WebdavResource array
	 */
	public WebdavResource[] ls(String path) throws NotExistsResourceException, ApplicationException {		
		if (this.resource == null)
			throw new ApplicationException();
		else {
			try {
				this.resource.setPath(path);
				Vector ressources = new Vector();
				WebdavResource[] liste = this.resource.listWebdavResources();
				if (this.resource.getStatusCode()==404)
					throw new NotExistsResourceException();
				return liste;
			}
			catch(IOException e) {
				LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::ls:"+e);
				throw new ApplicationException();
			}
			catch(HttpException e) {
				LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::ls:"+e);
				throw new ApplicationException();
			}
			}       	
	}
	
	/**
	 * Upload a file to the path given in argument
	 * @param inputFile the input file
	 * @param path the path
	 * @return boolean true if no problem, else false
	 * @throws ApplicationException
	 * @throws ServerException
	 * @throws NotExistsResourceException
	 * @throws StillExistsException
	 * @throws BadFormatException
	 * @throws UploadException
	 */
	public boolean upload(MultipartDataSource inputFile, String path) throws ApplicationException, ServerException, NotExistsResourceException, StillExistsException, BadFormatException, UploadException, EmptyFileOnUploadException,OverQuotaException{ 

		try {			
			
			// get the file name		      		      
			String fileName = inputFile.getName();
			
			// the file to upload to the webdav			           					
            String cible = path+fileName;                                            
            
            // check if the file still exists 
            boolean exist = isExist(fileName, path);
            if ( exist ) {
            	LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::upload : file already exist");
            	throw new StillExistsException();
            }
            else {       
            	// check if the filename contains fobiden characters
            	boolean wellFormed = DavControl.isWellFormed(fileName);
            	if ( !wellFormed ) {
                	LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::upload : name not well formed");
                	throw new BadFormatException();
            	}
            	// we can send
            	else {            		
            		this.resource.setPath(cible);
            	 
            		boolean answer = this.resource.putMethod(inputFile.getInputStream());
            		int httpCode = this.resource.getStatusCode();
            		if (httpCode==507)throw new OverQuotaException(httpCode);
            		
            		if ((httpCode>=400) && (httpCode<600) ) answer=false;
            		
            		if (answer) {
            			
            			// we check if the uploaded file is empty
            			if (fileIsEmptyOnServer(fileName, path)) {
            				// we delete the file
            				delete(fileName, path);
            				
            				// we throw a new Exception
            				throw new EmptyFileOnUploadException();
            			}
            			else {           				
            				return true;
            			}            			
            		}
            		else {
                    	//LogService.log(LogService.ERROR, "DavAccessImpl::upload ");
                    	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::upload http code : "+ httpCode);
            			throw new UploadException(httpCode);                			
            		}     
            	}
            }
        }
        catch (IOException ex) {  
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::upload "+ex);
            throw new ServerException();
        }
        catch (HttpException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::upload "+ex);
            throw new ServerException();
        } 
        catch (DeleteException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::upload "+ex);
            throw new UploadException();
        } 
	}
	
	
	/**
	 * Check if a file is empty on the server
	 * @param fileName the name of the file to check
	 * @param path the path where is the file on the server
	 * @return true if the file is empty on the server, false else
	 */
	private boolean fileIsEmptyOnServer(String fileName, String path) throws ApplicationException {
		// we keep the path at this moment
		String oldPath = this.resource.getPath();
		
		try {
			String pathAndName = path+fileName;
			
			// check if this is a valid resource
			this.resource.setPath(pathAndName);						
			if (this.resource.exists()) {
				int length = (int)resource.getGetContentLength();							
				if (length == 0)
					return true;
				else return false;
			}
			else return false;
		}
		catch (Exception e) {			
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::fileIsEmptyOnServer:"+e);
			throw new ApplicationException();
		}		   
	}
	
	
	/**
	 * Delete the resource given in argument
	 * @param file file/directory to delete
	 * @param path path where is file/directory
	 * @return boolean true if well done else false
	 * @throws ApplicationException
	 * @throws DeleteException 
	 */
	public boolean delete(String file, String path) throws ApplicationException, DeleteException {
		try {			 
            String cible = path+file;
			
            // check if is a directory
            boolean isDir = isDirectory(file, path);
            if (isDir)
            	cible = cible+"/";            
            
            boolean answer = this.resource.deleteMethod(cible);
    		int httpCode = this.resource.getStatusCode();
    		if ((httpCode>=400) && (httpCode<600) ) answer=false;
    		if (answer) { 
                return true;
            } else {
            	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::delete http code : "+ httpCode);
    			throw new DeleteException(httpCode);
            }
        }
		catch (IOException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::delete : "+ex);
        	throw new DeleteException();
        }
        catch (HttpException ex) {
         	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::delete : "+ex);
        	throw new DeleteException();
        }              
	}
	
	
	/**
	 * Rename the resource
	 * @param oldName file/directory to rename
	 * @param newName new name of the file/directory
	 * @param path path where is file/directory
	 * @return boolean true if well done else false
	 * @throws StillExistsException
	 * @throws BadFormatException
	 * @throws ApplicationException
	 * @throws RenameException
	 */
	public boolean rename(String oldName, String newName, String path) throws StillExistsException, BadFormatException, ApplicationException, RenameException {
		try {
			//check if file exist on dav Server
            boolean exist = isExist(newName, path);
            if ( exist ) {
            	LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::rename : file already exist");
            	throw new StillExistsException();            	
            }
            else {          
            	// check if the filename contains fobiden characters
            	boolean wellFormed = DavControl.isWellFormed(newName);
            	if ( !wellFormed ) { 
            		LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::rename : name not well formed");
                	throw new BadFormatException();
            	}
            	// we can rename
            	else {            		
            		String cible = path+oldName;
			
            		// check if is a directory
            		boolean isDir = isDirectory(oldName, path);
            		if (isDir )
            			cible = cible+"/"; 
            		
            		this.resource.setPath(cible);
			
            		// we rename            
            		cible = path+newName;
            		boolean answer = this.resource.moveMethod(cible);
             		int httpCode = this.resource.getStatusCode();
             		if ((httpCode>=400) && (httpCode<600) ) answer=false;
             		if (answer) { 
            			return true;
             		}
            		else{
            			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::rename http code : "+ httpCode);
            			throw new RenameException(httpCode);
            		}
            	}            
            }
		}
        catch (HttpException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::rename : "+ex);
        	throw new RenameException();
        }
        catch (IOException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::rename : "+ex);
        	throw new RenameException();
        }               
	}
	
	
	
	/**
	 * Create the directory given in argument
	 * @param file file/directory to delete
	 * @param path path where is file/directory
	 * @return boolean true if well done else false
	 * @throws ApplicationException
	 * @throws CreateDirectoryException
	 * @throws StillExistsException
	 * @throws BadFormatException
	 */
	public boolean createDir (String name, String path) throws ApplicationException, CreateDirectoryException, StillExistsException, BadFormatException{
		try {
			//look if a file with the same name exist
			boolean exist = isExist(name, path);
            if ( exist ) {
            	LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::createDir : file already exist");
            	throw new StillExistsException();           	
            }
            else {          
            	// check if the file is well-formed
            	boolean wellFormed = DavControl.isWellFormed(name);
            	if ( !wellFormed ) {
            		LogService.log(LogService.INFO, "CWebdav::DavAccessImpl::createDir : name not well formed");
                	throw new BadFormatException();
            	}
            	// we can create the directory
            	else {            		
            		this.resource.setPath(path);  
               		
               		boolean answer = this.resource.mkcolMethod(path+name);
               		int httpCode = this.resource.getStatusCode();
            		if ((httpCode>=400) && (httpCode<600) ) answer=false;
            		if (answer)
            			return true;
            		else {
                    	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::createDir : code http : "+httpCode);
                    	throw new CreateDirectoryException(httpCode);
            		}
            	}
            }
		}
        catch (HttpException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::createDir : "+ex);
        	throw new CreateDirectoryException();
        }
        catch (IOException ex) {
         	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::createDir : "+ex);
        	throw new CreateDirectoryException();
        }          
	}
	
	
	/**
	 * Check if we can paste here
	 * @param clipboardPath the path where is the clipboad
	 * @param clipboard the containt of the clipboard
	 * @param curentDirPath the path of the curent dir
	 * @return true if ok excpetion else
	 * @throws ApplicationException
	 * @throws PasteInChildDirectoryException
	 * @throws PasteWithSameNameException
	 * @throws PasteDeletedResourceException
	 */
	public boolean canPaste(String clipboardPath, Vector clipboard, String curentDirPath) throws ApplicationException, PasteInChildDirectoryException, PasteWithSameNameException, PasteDeletedResourceException {
		
		// we check if there is a resource with the same name in the curent dir
		for (int i=0; i<clipboard.size(); i++) {
			String resourceName = (String)clipboard.elementAt(i);
			boolean existe = isExist(resourceName, curentDirPath);
			if (existe)
				throw new PasteWithSameNameException();			
		}
		
		// we check if we try to paste a resource in its child
		for (int i=0; i<clipboard.size(); i++) {
			String resourceName = (String)clipboard.elementAt(i);
			if (curentDirPath.startsWith(clipboardPath+resourceName+"/")) {				
				throw new PasteInChildDirectoryException();
			}					
		}
		
		// we check if one of the selected resource has been deleted
		for (int i=0; i<clipboard.size(); i++) {
			String resourceName = (String)clipboard.elementAt(i);
			if (!isExist(resourceName, clipboardPath))
				throw new PasteDeletedResourceException();					
		}
		
		return true;
	}
	
	
	
	/**
	 * Copy method
	 * @param resource resource to copy
	 * @param fromPath path where is the resource
	 * @param toPath path to copy the resource
	 * @return boolean true if well done else false
	 * @throws ApplicationException
	 * @throws CopyException 
	 */
	public boolean copy(String resource, String fromPath, String toPath) throws CopyException, ApplicationException,OverQuotaException {
		try {						
			
			// check if resource is a directory   
			String realResource = resource;
			boolean isDir = isDirectory(resource, fromPath);
            if (isDir)
            	realResource = realResource+"/"; 			
			
			// absolute path of the resource
			String absoluteFromPath = fromPath+realResource;
			
			// absolute path to copy the resource
			String absoluteToPath = toPath;
			if (!toPath.endsWith("/"))
				absoluteToPath += "/";
			absoluteToPath += realResource;
            
			// fix the path
            this.resource.setPath(absoluteFromPath);
            
            boolean answer = this.resource.copyMethod(absoluteToPath);
       		int httpCode = this.resource.getStatusCode();
       		
       		if (httpCode==507)throw new OverQuotaException(httpCode);
    		
    		if ((httpCode>=400) && (httpCode<600) ) answer=false;
    		
    		if (answer)
    			return true;
    		else {
            	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::copy : code http : "+httpCode);
            	throw new CopyException(httpCode);
    		}
        }
        catch (HttpException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::copy : "+ex);
        	throw new CopyException();
        }
        catch (IOException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::copy : "+ex);
        	        	throw new CopyException();
        }         
	}
	
	
	
	
	
	/**
	 * Move method
	 * @param resource resource to move
	 * @param fromPath path where is the resource
	 * @param toPath path to move the resource
	 * @return boolean true if well done else false
	 * @throws ApplicationException
	 * @throws MoveException
	 */
	public boolean move(String resource, String fromPath, String toPath) throws ApplicationException, MoveException {
		try {						
			
			// check if resource is a directory   
			String realResource = resource;
			boolean isDir = isDirectory(resource, fromPath);
            if (isDir)
            	realResource = realResource+"/"; 			
			
			// absolute path of the resource
			String absoluteFromPath = fromPath+realResource;
			
			// absolute path to copy the resource
			String absoluteToPath = toPath;
			if (!toPath.endsWith("/"))
				absoluteToPath += "/";
			absoluteToPath += realResource;
            
			// fix the path
			this.resource.setPath(absoluteFromPath);

            boolean answer = this.resource.moveMethod(absoluteToPath);
       		int httpCode = this.resource.getStatusCode();
    		if ((httpCode>=400) && (httpCode<600) ) answer=false;
    		if (answer)
    			return true;
    		else {
            	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::move : code http : "+httpCode);
            	throw new MoveException(httpCode);
    		}
    		
        }
        catch (HttpException ex) {
         	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::move : "+ex);
        	throw new MoveException();
        }
        catch (IOException ex) {
        	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::move : "+ex);
        	throw new MoveException();
        }               
	}
	
	
	/*******************
	 * Download methods
	 *******************/
	
	
	/** 
	 * Prepare the file whiwh will be downloaded
	 * 
	 * @see org.esupportail.portal.channels.CWebdav.provider.DavAccess#setFileToDownload(java.lang.String, java.lang.String)
	 */
	public boolean setFileToDownload(String name,String path){
		
		try {
			this.init(this.uri,this.login,this.password);
		} catch (MalformedURLException e2) {
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::setFileToDownload MalformedURLException: "+e2);
		}
        String monFic = httpURL+path+name;
		try{
			fileToDownload = new WebdavResource(monFic);
		}catch(IOException e){
	     	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::setFileToDownload IOException: "+e);			
		}catch(HttpException e){
	     	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::setFileToDownload HttpException: "+e);						
		}
		return fileToDownload.getExistence();
	}
	
	
	/** 
	 * Search content type of the file whiwh will be downloaded
	 * 
	 * @see org.esupportail.portal.channels.CWebdav.provider.DavAccess#getContentType()
	 */
	public String getContentType() {
		
		String content=null;
		
		if (fileToDownload!=null){
			content= fileToDownload.getGetContentType();
		}
	    return content;
	}
	
	
	
	

	/** 
	 * Real download
	 * WARNING : the method "fileToDownload.getMethodData()" create a tempory file
	 * 			 which be delete when the JVM will be stoped
	 * 
	 * @see org.esupportail.portal.channels.CWebdav.provider.DavAccess#downloadData(java.io.OutputStream, java.lang.String, java.lang.String)
	 */
	public InputStream getInputStream() {
	        try {
				return fileToDownload.getMethodData();
			} catch (HttpException e) {
				LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::getInputStream: HttpException : "+e);
			}catch(IOException e){
		     	LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::getInputStream : IOException : "+e);			
			}
			return null;
	}
	
	
	
	
	/************************
	 * Webdav control methods
	 ************************/
	
	/**
	 * Check is the resource exists
	 * @param resourceName name of the file/directory to verify
	 * @param path path to find the file/directory
	 * @return true if file/directory exists else false
	 * @throws IOException
	 * @throws HttpException
	 */
	public boolean isExist(String resourceName, String path) throws ApplicationException {
		
		// we keep the path at this moment
		String oldPath = this.resource.getPath();
		
		try {
			String pathAndName = path+resourceName;
			
			// check if this is a valid resource
			this.resource.setPath(pathAndName);						
			if (this.resource.exists()) {
				return true;
			}
			else {
				this.resource.setPath(pathAndName+"/");
				if (this.resource.exists()) {
					return true;
				}
				else {
					return false;
				}
			}							
		}
		catch (Exception e) {			
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::isExist:"+e);
			throw new ApplicationException();
		}		
		
	}//end isExist
	
	
	
	
	/**
	 * Check if the resource named name is a directory in the path given
	 * @param name the name of the resource
	 * @param path the path
	 * @return true if directory, false else
	 * @throws ApplicationException
	 */
	public boolean isDirectory(String name, String path) throws ApplicationException {
		
		// we keep the path at this moment
		String oldPath = this.resource.getPath();
		
		try {
			
			this.resource.setPath(path+name);
			if (this.resource.isCollection()) {
				return true;
			}
			else {
				this.resource.setPath(path+name+"/");
				if (this.resource.isCollection()) {
					return true;
				}
				else {
					return false;
				}
			}				
		}
		catch (IOException e) {			
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::isDirectory:"+e);
			throw new ApplicationException();
		}	
		catch (HttpException e) {			
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::isDirectory:"+e);
			throw new ApplicationException();
		}	
	}
	
	
	
	
	/**
	 * Check if the directory named dir is empty or not. We suppose that dir has been checked as a directory before
	 * @param dir the name of the directory
	 * @param path the path
	 * @return true if empty, false else
	 * @throws ApplicationException
	 */
	public boolean isEmpty (String dir, String path) throws ApplicationException, NotExistsResourceException {		
				
		try {
			
			String pathToList;
			
			// we construct the path
			if (!dir.endsWith("/"))
				pathToList = path+dir+"/";
			else pathToList = path+dir;
			
			// we count the number of resources
			if (ls(pathToList).length > 0){
				return false;
			}
			else {
				return true;
			}//end if							
		}
		catch (ApplicationException e) {
			LogService.log(LogService.ERROR, "CWebdav::DavAccessImpl::isEmpty:"+e);
			throw new ApplicationException();
		}		
	}	
	
	
	
	
	
	
	
	
}
