/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/

package org.esupportail.portal.channels.CWebdav.renderXml;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Calendar;
import java.util.Vector;
import org.apache.webdav.lib.WebdavResource;
import org.esupportail.portal.channels.CWebdav.Constants; 
import org.esupportail.portal.channels.CWebdav.Config;
import org.esupportail.portal.channels.CWebdav.exception.PropertiesException;
import org.esupportail.portal.channels.CWebdav.exception.WebdavException;
import org.esupportail.portal.channels.CWebdav.provider.DavAccess;
import org.esupportail.portal.channels.CWebdav.provider.DavControl;
import org.jasig.portal.services.LogService;

/** 
 * Id: RenderXmlShowCurrentDir.java,v1.0 15 avr. 2004
 * Copyright (c) 2004 Esup Portail (www.esup-portail.org)
 * Classes: RenderXmlShowCurrentDir
 * Original Author: Yohan Colmant
 * Get the XML to show the curent dir
 */
public class RenderXmlShowCurrentDir {
	
	/**
	 * Return the XML representing the curent dir
	 * @param access the object used to access the webdav
	 * @param curentDir the content of the curent directory
	 * @param error the error message
	 * @param information this information message
	 * @return the xml
	 */
	public static StringBuffer getXml(DavAccess access, Vector curentDir, WebdavException error, String information) {				
		StringBuffer xml = new StringBuffer();
				
		int erreurInterne = -1;
						
		// the content of the directory
		String path = DavControl.getCurentPath(curentDir);		
		try {
			
			// the action bar
			xml.append(getXmlAction());			
			
			// construct the ressources
			WebdavResource[] ressources;
			
			if (access!=null)
				ressources = access.ls(path);
			else ressources = new WebdavResource[0];
			
			// the content of the directory	
			xml.append(getXmlContent(ressources));							
						
		}
		catch(WebdavException e) {			
			erreurInterne = e.getErrorCode();
			LogService.log(LogService.ERROR, "CWebdav::RenderXmlShowCurrentDir::getXml:"+e);
		}		
		
		// the error bar
		xml.append(getXmlError(error, erreurInterne));
		
		// the information bar
		xml.append(getXmlInformation(information));		
		
		// the navigation bar			
		xml.append(getXmlNavigation(curentDir));										
			
		return xml;
	}
	
	
	
	
	/**
	 * Get the XML for the navigation bar
	 * @param curentDir
	 * @return
	 */
	private static StringBuffer getXmlNavigation(Vector curentDir) {
		StringBuffer xmlTemp = new StringBuffer();
		
		xmlTemp.append("<NAVIGATION>");
		for (int i=0; i<curentDir.size(); i++) {			
			xmlTemp.append("	<DIR label=\""+((String)curentDir.elementAt(i)).replaceAll("\\~","")+"\" link=\""+i+"\" />");
		}
		xmlTemp.append("</NAVIGATION>");
		
		return xmlTemp;		
	}
	
	
	
	
	/**
	 * Get the XML for the error bar
	 * @param error the error
	 * @param internalError if there is an internal error
	 * @return
	 */
	private static StringBuffer getXmlError(WebdavException error, int internalError) {
		StringBuffer xml = new StringBuffer();
		
		xml.append("<ERRORS>");
		if ( error!=null )
			xml.append("<ERROR httpCode=\""+error.getHttpErrorCode()+"\" channelCode=\""+error.getErrorCode()+"\" />");
		else {
			if (internalError!=-1)
				xml.append("<ERROR channelCode=\""+internalError+"\" />");
		}
		xml.append("</ERRORS>");
		return xml;
	}
	
	
	
	
	/**
	 * Get the XML for the info bar
	 * @param information the information message code
	 * @return
	 */
	private static StringBuffer getXmlInformation(String information) {
		StringBuffer xml = new StringBuffer();
		
		if (information!=null)
			xml.append("<INFORMATION>"+information+"</INFORMATION>");
		
		return xml;
	}
	
	
	
	
	/**
	 * Get the XML for the action bar
	 * @return
	 * @throws PropertiesException
	 */
	private static StringBuffer getXmlAction() throws PropertiesException {
		StringBuffer xmlTemp = new StringBuffer();
		
		xmlTemp.append("<ACTIONS>");
		xmlTemp.append("<ACTION label=\""+Config.getInstance().getDelete()+"\" mode=\""+Constants.CONFIRM_DELETE_MODE+"\" />");
		xmlTemp.append("<ACTION label=\""+Config.getInstance().getRename()+"\" mode=\""+Constants.SET_NAME_FOR_RENAME_MODE+"\" />");
		xmlTemp.append("<ACTION label=\""+Config.getInstance().getNewDir()+"\" mode=\""+Constants.SET_NAME_FOR_CREATE_DIR_MODE+"\" />");
		xmlTemp.append("<ACTION label=\""+Config.getInstance().getCopy()+"\" mode=\""+Constants.SET_CLIPBOARD_FOR_COPY_MODE+"\" />");
		xmlTemp.append("<ACTION label=\""+Config.getInstance().getMove()+"\" mode=\""+Constants.SET_CLIPBOARD_FOR_MOVE_MODE+"\" />");
		xmlTemp.append("</ACTIONS>");
		
		return xmlTemp;		
	}
	
	
	
	
	/**
	 * Get the XML for the content
	 * @param res a WebdavResource
	 * @return
	 * @throws PropertiesException
	 */
	private static StringBuffer getXmlContent(WebdavResource[] ressources) throws PropertiesException {
				
		StringBuffer xmlDirectory = new StringBuffer();
		StringBuffer xmlFile = new StringBuffer();
				
		for (int i=0; i<ressources.length; i++) {
			WebdavResource res = ressources[i];
			
		if (res.isCollection()) {
			try {
				xmlDirectory.append("<DIRECTORY label=\""+res.getDisplayName()+"\" link=\""+URLEncoder.encode(res.getDisplayName(),"ISO-8859-1")+"\"/>");
			} catch (UnsupportedEncodingException e) {
				LogService.log(LogService.ERROR, "CWebdav::RenderXmlShowCurrentDir::getXml:"+e);	
			}
		}
		else {
			// we check if the file can be showed
			String name = res.getDisplayName();
			Vector invisibles = Config.getInstance().getInvisibleRegexp();                   
			boolean autorise = true;
			for (int j=0; j<invisibles.size(); j++) {
				String inv = (String)invisibles.elementAt(j);    
				
				if (name.matches(inv))                           
					autorise = false;                       
               	}    	           
               	if (autorise) {   
               		// date format
               		long d = res.getGetLastModified();        	          
               		Calendar cal = Calendar.getInstance();
               		
               		cal.setTimeInMillis(d);
               		String date = cal.get(Calendar.DATE)+"";
               		if (date.length()==1)
               			date = "0"+date;
               		String month = (cal.get(Calendar.MONTH)+1)+"";
               		if (month.length()==1)
               			month = "0"+month;
               		String year = cal.get(Calendar.YEAR)+"";	                       
               		String hour = (cal.get(Calendar.HOUR_OF_DAY))+"";
               		
               		if (hour.length()==1)
               			hour = "0"+hour;
               		String minute = cal.get(Calendar.MINUTE)+"";
               		if (minute.length()==1)
               			minute = "0"+minute;
               		
               		// length format
               		long intLength = (long)res.getGetContentLength();	                   		      			
            		float floatLength = intLength;				
            		long divInt = (intLength /1024)/1024;		
            		
            		String length = "";
            		if (divInt > 0) {			
            			length = ""+(floatLength/1024)/1024;
            			int index = length.indexOf(".");					
            			if (length.length() > index+3) {
            				length = length.substring(0, index+3);
            			}						
            			length += " MB";
            		}
            		else {
            			divInt = (intLength /1024);			
            			if (divInt > 0) {			
            				length = ""+(floatLength/1024);
            				int index = length.indexOf(".");					
            				if (length.length() > index+3) {
            					length = length.substring(0, index+3);
            				}						
            				length += " KB";
            			}
            			else {			
            				length = ""+(floatLength);				
            				int index = length.indexOf(".");			
            				if (length.length() > index+3) {
            					length = length.substring(0, index+3);
            				}
            				length += " B";
            				}
            			}

               		try {
               			
           		xmlFile.append("<FILE ");
           		xmlFile.append("label=\""+name+"\" ");
           		xmlFile.append("label_download=\""+URLEncoder.encode(name,"ISO-8859-1")+"\" ");
           		xmlFile.append("mime=\""+res.getGetContentType()+"\" ");
           		xmlFile.append("weight=\""+length+"\" ");
           		xmlFile.append("dateLastUpdate=\""+date+"\" ");
           		xmlFile.append("monthLastUpdate=\""+month+"\" ");
           		xmlFile.append("yearLastUpdate=\""+year+"\" ");
           		xmlFile.append("hourLastUpdate=\""+hour+"\" ");
           		xmlFile.append("minuteLastUpdate=\""+minute+"\" />");
				} catch (UnsupportedEncodingException e1) {
					LogService.log(LogService.ERROR, "CWebdav::RenderXmlShowCurrentDir::getXML : "+ e1);
				}
				 
               }
           }
		}		
		
		StringBuffer xml = new StringBuffer();
		xml.append("<CONTENT>");										
		xml.append(xmlDirectory);
		xml.append(xmlFile);
		xml.append("</CONTENT>");
		return xml;
	}
	
	
}
